﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Microsoft.Win32;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;

namespace Dock
{
    /// <summary>
    /// Dock item
    /// </summary>
    [Serializable]
    public class DockItem : IComparer<DockItem>
    {
        #region Properties

        /// <summary>
        /// Get the index of this DockItem in the Dock
        /// </summary>
        public int Index
        {
            get;
            internal set;
        }

        /// <summary>
        /// Get or set the file name and path
        /// </summary>
        public string FileName
        {
            get;
            set;
        }

        /// <summary>
        /// Get the name of the file
        /// </summary>
        public string Name
        {
            get;
            private set;
        }

        /// <summary>
        /// Get the file extension
        /// </summary>
        public string FileExtension
        {
            get;
            private set;
        }

        /// <summary>
        /// Get the file extension description
        /// </summary>
        public string FileExtensionDesc
        {
            get;
            private set;
        }

        /// <summary>
        /// Get the MIME type for this file
        /// </summary>
        public string MimeType
        {
            get;
            private set;
        }

        /// <summary>
        /// Get the position of this drag item
        /// </summary>
        public int PositionX
        {
            get;
            set;
        }

        /// <summary>
        /// Get a description of the file for displaying in tooltips
        /// </summary>
        public string Tooltip
        {
            get
            {
                if (this.FileExtensionDesc == null)
                {
                    return Path.GetFileNameWithoutExtension(this.Name);
                }
                else
                {
                    return Path.GetFileNameWithoutExtension(this.Name) + "\n" + this.FileExtensionDesc;
                }
            }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Create a DockItem
        /// </summary>
        /// <param name="itemNumber"></param>
        /// <param name="fileName"></param>
        /// <param name="posX"></param>
        private DockItem(int itemNumber, string fileName, int posX)
        {
            // Update the properties
            this.Index = itemNumber;
            this.FileName = fileName;
            this.PositionX = posX;
            this.Name = Path.GetFileName(fileName);
            this.FileExtension = Path.GetExtension(fileName);
            this.FileExtensionDesc = Win32Ex.GetExtenstionDescription(this.FileExtension);
            this.MimeType = DockItem.GetMimeType(fileName);
        }

        #endregion

        #region Methods

        /// <summary>
        /// Create a new DockItem
        /// </summary>
        /// <param name="itemNumber"></param>
        /// <param name="fileName"></param>
        /// <param name="posX"></param>
        /// <returns></returns>
        public static DockItem Create(int itemNumber, string fileName, int posX)
        {
            // Create a new DockItem
            DockItem item = new DockItem(itemNumber, fileName, posX);

            // Copy the fileName to the Dock directory
            item.FileName = item.CopyFile(fileName);

            // Return the item
            return item;
        }

        /// <summary>
        /// Load a DockItem
        /// </summary>
        /// <returns></returns>
        public static DockItem Load(int itemNumber, string fileName, int posX)
        {
            // Return the item
            return new DockItem(itemNumber, fileName, posX);
        }

        /// <summary>
        /// Compare two instances of a DocItem
        /// </summary>
        /// <param name="item1"></param>
        /// <param name="item2"></param>
        /// <returns></returns>
        public int Compare(DockItem x, DockItem y)
        {
            if (x == null)
            {
                if (y == null)
                {
                    // If x is null and y is null, they're
                    // equal. 
                    return 0;
                }
                else
                {
                    // If x is null and y is not null, y
                    // is greater. 
                    return -1;
                }
            }
            else
            {
                // If x is not null...
                //
                if (y == null)
                // ...and y is null, x is greater.
                {
                    return 1;
                }
                else
                {
                    // ...and y is not null, compare the 
                    // lengths of the two strings.
                    //
                    int retval = x.Index.CompareTo(y.Index);

                    if (retval != 0)
                    {
                        // If the strings are not of equal length,
                        // the longer string is greater.
                        //
                        return retval;
                    }
                    else
                    {
                        // If the strings are of equal length,
                        // sort them with ordinary string comparison.
                        //
                        return x.Index.CompareTo(y);
                    }
                }
            }
        }

        /// <summary>
        /// Get the large icon for this file
        /// </summary>
        /// <returns></returns>
        public Icon GetLargeIcon()
        {
            return Win32Ex.GetIcon(this.FileName, SHGFI.Icon | SHGFI.LargeIcon);
        }

        /// <summary>
        /// Get the small icon for this file
        /// </summary>
        /// <returns></returns>
        public Icon GetSmallIcon()
        {
            return Win32Ex.GetIcon(this.FileName, SHGFI.Icon | SHGFI.SmallIcon);
        }

        /// <summary>
        /// Open the file
        /// </summary>
        /// <returns></returns>
        public Process Open()
        {
            if (File.Exists(this.FileName))
            {
                return Process.Start(this.FileName);
            }

            throw new FileNotFoundException("File not found", this.FileName);
        }

        /// <summary>
        /// Delete the file from the disk
        /// </summary>
        public void Delete()
        {
            if (File.Exists(this.FileName))
            {
                File.Delete(this.FileName);
            }
        }

        /// <summary>
        /// Get a string representation of this DockItem (File name without the extension)
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return Path.GetFileNameWithoutExtension(this.Name);
        }
        
        #endregion

        #region Private methods

        /// <summary>
        /// Save a copy of the file
        /// </summary>
        /// <returns>The file path of the copied file</returns>
        private string CopyFile(string sourceFile)
        {
            // Create the path to save the file
            string filePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), Application.ProductName);
            
            // Create the fileName path if it doesn't exist
            if (!Directory.Exists(filePath))
            {
                Directory.CreateDirectory(filePath);
            }

            // Create the file name
            string fileName = Path.Combine(filePath, this.Name);

            // Copy the file
            File.Copy(sourceFile, fileName, true);

            // Reset the file name
            return fileName;
        }

        #endregion

        #region Static methods

        /// <summary>
        /// Get the mime content type of the file
        /// </summary>
        /// <param name="fileName">Name of the file to get the content type of</param>
        /// <returns></returns>
        public static string GetMimeType(string fileName)
        {
            // Defualt mime-type
            string mimeType = "application/octetstream";

            // Get the extension of the fileName specified
            string ext = System.IO.Path.GetExtension(fileName).ToLower();

            // Open the registry key
            RegistryKey regKey = Registry.ClassesRoot.OpenSubKey(ext);

            // Get the content type
            if (regKey != null && regKey.GetValue("Content Type") != null)
            {
                mimeType = regKey.GetValue("Content Type").ToString();
            }

            // Return the mime value
            return mimeType;
        }

        #endregion
    }
}
